using System.Reflection;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;

namespace FitConnect.Encryption;

public static class JsonHelper {
    public static bool ValidateJsonSchema(string schema, string json, ILogger? logger = null) {
        var jSchema = Newtonsoft.Json.Schema.JSchema.Parse(schema);
        return JObject.Parse(json).IsValid(jSchema);
    }

    /// <summary>
    ///     Verify the metadata hash and content to fit the schema
    /// </summary>
    /// <param name="metadataString"></param>
    /// <param name="logger">Optional logger, to print errors to logger</param>
    /// <returns></returns>
    public static bool VerifyMetadata(string metadataString, ILogger? logger = null) {
        var schemaString = LoadContentOfResource("metadata.schema.json");
        var schema = NJsonSchema.JsonSchema.FromJsonAsync(schemaString).Result;
        var result = schema.Validate(metadataString).ToList();
        logger?.LogWarning("Validation failed with {Errors}",
            result.Aggregate("", (a, b) => a + "\n\t" + b));
        return result.Count == 0;
    }


    private static string LoadContentOfResource(string resourceName) {
        var assembly = Assembly.GetExecutingAssembly();
        var fullQualifiedName = $"{assembly.GetName().Name}.{resourceName}";
        var resourceStream = assembly.GetManifestResourceStream(fullQualifiedName);
        if (resourceStream == null)
            throw new Exception($"Resource {fullQualifiedName} not found");
        var reader = new StreamReader(resourceStream);
        return reader.ReadToEnd();
    }

    public static bool ValidateJsonSchema(Uri schema, string jsonData) {
        var schemaResponse = new HttpClient()
            .GetAsync(schema)
            .Result;

        return ValidateJsonSchema(schemaResponse.Content.ReadAsStringAsync().Result, jsonData);
    }
}
